/** ----------------------------------- -o-
 *  DebugLabel.java
 *
 * Lightweight means to track order of execution.
 * All output is prefiexed with "<tag> <class>.<method>.<thread>".
 *
 * Always balance b/sm/arglist with e/ms so label context stack works out ok.
 *   (st doesn't change anything.)
 * 
 * CLASSES/METHODS--
 *   st sm ms / pc pcd pt
 *     pr i err merr fmerr d / arglist / b e level no / hi hir
 *   void ASSERT()
 *
 * @version 0.1    CREATED: Thu Sep 14 00:00:00 PDT 2009
 * @author  dreeder@danaprajna.com
 */ 							// -o-
package com.danaprajna;

import java.util.Stack;
import com.cycling74.max.*; 


public class DebugLabel extends MaxObject 	
{ 							
  //
  // variables 						// -o-
  //
  public  String   dc 		= ""; 		// class string prefix
  public  String   dm 		= ""; 		// method string prefix
  private int      tid 		= -1; 		// thread id if applicable

  private boolean debugOn 	    = false;
  private boolean stdoutToPost      = true; 	// output to Max and/or stdout
  private boolean stdoutToTerminal  = false;

  private Stack<String> context = new Stack<String>();
    // to maintain labelling context between {b/sm e/ms} pairs



  // constructors 					// -o-
  //
  public DebugLabel(String s)  		  { dc=dm=s; }
  public DebugLabel(String s, boolean b)  { dc=dm=s; debugOn=b; }
  public DebugLabel(String s, boolean b, boolean use_post, boolean use_terminal)
    { dc=dm=s; debugOn=b; 
      stdoutToPost=use_post; stdoutToTerminal=use_terminal; }



  // module specific debug 				// --o- 
  //
  public DebugLabel st(int id)    { tid = id; 			return(this); }
  public DebugLabel sm(String m)  { dm = m; pt("NA", "push"); 	return(this); }
  public DebugLabel ms() 	  { pt("NA", "pop"); 		return(this); }

  public void pc(String s) { 
      if (stdoutToTerminal)  	{ System.out.println(s); }
      if (stdoutToPost)  	{ post(s); }
    }  
  private void pcd(String s)  { if (debugOn) { pc(s); } }


  private String mkpt() {
      String in = "";

      if (null==dc && null==dm)  	{ in += ""; }
      else if (null!=dc && null!=dm)  	{ in += dc + "." + dm; }
      else  				{ in += (null==dc) ? "."+dm : dc+"."; }
      if (-1 != tid) 			{ in += "."+tid; }

      return(in);
    }

  private String pt(String dtype, String stack_action) {
      String prefix 	= dtype + "\t\t\t\t\t";
      String out 	= "";

      if      ("pop" == stack_action) 	    { out += context.pop(); }
      else if ("peek" == stack_action) { 
          if (context.empty())  	    { out += mkpt(); }
	  else 				    { out += context.peek(); }
	}
      else  /* "push" new label */ 	    { out += context.push(mkpt()); }

      return(prefix + out + " :: ");
    }

  private String pt() 		{ return(pt("DEBUG", "peek")); }
  private String pt(String s)  	{ return(pt(s, "peek")); }
  private String ptu()  	{ return(pt("DEBUG", "push")); }
  private String pto()  	{ return(pt("DEBUG", "pop")); }
  

  public DebugLabel pr(String s)   { pc( pt("-") + s ); 	 return(this); }
  public DebugLabel pr(Atom[] a)   { return(pr(Atom.toOneString(a))); }
  public DebugLabel pr(Atom a) 	   { return(pr(a.toString())); }
  public DebugLabel pr(int i) 	   { return(pr(i + "")); }
  public DebugLabel pr(boolean b)  { return(pr(b?"true":"false")); }
  public DebugLabel pr() 	   { return(pr("")); }

  public DebugLabel i(String s)   { pc( pt("INFO") + s ); 	 return(this); }
  public DebugLabel i(Atom[] a)   { return(i(Atom.toOneString(a))); }
  public DebugLabel i(Atom a)  	  { return(i(a.toString())); }
  public DebugLabel i(int i)  	  { return(i(i + "")); }
  public DebugLabel i(boolean b)  { return(i(b?"true":"false")); }
  public DebugLabel i()  	  { return(i("")); }

  public DebugLabel d(String s)   { pcd( pt("DEBUG") + s ); 	 return(this); }
  public DebugLabel d(Atom[] a)   { return(d(Atom.toOneString(a))); }
  public DebugLabel d(Atom a)     { return(d(a.toString())); }
  public DebugLabel d(int i)      { return(d(i + "")); }
  public DebugLabel d(boolean b)  { return(d(b?"true":"false")); }
  public DebugLabel d()  	  { return(d("")); }

  public DebugLabel err(String s)   { pcd( pt("ERR") + s );  	return(this); }
  public DebugLabel err(Atom[] a)   { return(err(Atom.toOneString(a))); }
  public DebugLabel err(Atom a)     { return(err(a.toString())); }
  public DebugLabel err(int i)      { return(err(i + "")); }
  public DebugLabel err(boolean b)  { return(err(b?"true":"false")); }
  public DebugLabel err()  	    { return(err("")); }

  public DebugLabel merr(String s)   
    { error("ERR \t\t\t\t\t" + s); 				return(this); }
  public DebugLabel merr(Atom[] a)   { return(merr(Atom.toOneString(a))); }
  public DebugLabel merr(Atom a)     { return(merr(a.toString())); }
  public DebugLabel merr(int i)      { return(merr(i + "")); }
  public DebugLabel merr(boolean b)  { return(merr(b?"true":"false")); }
  public DebugLabel merr()  	     { return(merr("")); }
 
  public void fmerr(String s)   { error("FATAL \t\t\t\t\t" + s); bail(""); }
  public void fmerr(Atom[] a)   { fmerr(Atom.toOneString(a)); }
  public void fmerr(Atom a)     { fmerr(a.toString()); }
  public void fmerr(int i)      { fmerr(i + ""); }
  public void fmerr(boolean b)  { fmerr(b?"true":"false"); }
  public void fmerr() 	        { fmerr(""); }


  public DebugLabel arglist(Atom[] a) 		             {        pc(pt() + 
      "ARGS(" + a.length + ")\t\t" + Atom.toOneString(a)); 
      return(this); }
  public DebugLabel arglist(String m, Atom[] a)              { sm(m); pc(pt() +
      "ARGS(" + a.length + ")\t\t" + Atom.toOneString(a)); 
      return(this); }
  public DebugLabel arglist(String m, String msg, Atom[] a)  { sm(m); pc(pt() + 
      msg + "(" + a.length + ")\t\t" + Atom.toOneString(a)); 
      return(this); }


  public void b()  		     {                pcd( ptu() + "BEGIN"); }
  public void b(String m)  	     { sm(m);         pcd( ptu() + "BEGIN"); }
  public void b(String m, int id)    { sm(m); st(id); pcd( ptu() + "BEGIN"); }
  public void b(String m, Atom[] a)  { sm(m);         b(); arglist(a); }
  public void b(Atom[] a)  	     { 		      b(); arglist(a); }

  public void e()  	   { pcd( pto() + "END" ); }
  public void e(Atom a)    { pcd( pto() + "END (" + a + ")"); }
  public void e(String s)  { e(Atom.newAtom(s)); }
  public void e(double d)  { e(Atom.newAtom(d)); }
  public void e(float f)   { e(Atom.newAtom(f)); }
  public void e(int i)     { e(Atom.newAtom(i)); }
  public void e(boolean b) { e(Atom.newAtom(b)); }
 
  public boolean  level()  	    { return(debugOn); } 	// get/set
  public boolean  level(boolean b)  { debugOn = b; return(debugOn); }
  public DebugLabel  no()      	    { debugOn = false; return(this); }



  // misc 						// -o-
  //
  private int highCount = 0;
  public void hir()  		{ highCount = 0; hi(); }
  public void hir(String s)  	{ highCount = 0; hi(s); }
  public void hi() 		{ hi(""); }
  public void hi(String s) 
    { String space = "                         ";
      pc(pt() + space + space + "hi " + highCount++ + "   " + s); }

  public boolean ASSERT(boolean b)  { return(ASSERT(b, "")); }
  public boolean ASSERT(boolean b, String s)
    { if (b)  { 			     return(true);  } 
      else    { merr("ASSERT FAILED: " + s); return(false); } }

}  // public class DebugLabel extends MaxObject


